@@ -4,25 +4,49 @@
 
 #if PY_MAJOR_VERSION >= 3
     #define PYBYTES_FSAS PyBytes_FromStringAndSize
+    #define PyInt_CheckExact(pyobj) 0
+    #define PyInt_AsLong(pyobj) 0
 #else
     #define PYBYTES_FSAS PyString_FromStringAndSize
 #endif
 
+#if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION <= 4
+    #define Py_ssize_t ssize_t
+#endif
 
 static PyObject *
 python_compress(PyObject *self, PyObject *args) {
     char *input, *output;
     Py_ssize_t inlen;
-    long outlen = -1;
+    PyObject *pyoutlen = Py_None;
+    long outlen;
     PyObject *result;
 
-    if (!PyArg_ParseTuple(args, "s#|l", &input, &inlen, &outlen))
+    if (!PyArg_ParseTuple(args, "s#|O", &input, &inlen, &pyoutlen))
         return NULL;
 
-    if (outlen < 0) outlen = inlen - 1;
-    if (inlen == 1) outlen++;
+    if (pyoutlen == Py_None)
+        outlen = inlen - 1;
+    else if (PyInt_CheckExact(pyoutlen))
+        outlen = PyInt_AsLong(pyoutlen);
+    else if (PyLong_CheckExact(pyoutlen))
+        outlen = PyLong_AsLong(pyoutlen);
+    else {
+        PyErr_SetString(PyExc_TypeError, "max_len must be an integer");
+        return NULL;
+    }
 
-    output = (char *)malloc(outlen);
+    if (inlen == 1) outlen++; /* work around for what looks like a liblzf bug */
+    if (outlen <= 0) {
+        PyErr_SetString(PyExc_ValueError, "max_len must be > 0");
+        return NULL;
+    }
+
+    output = (char *)malloc(outlen + 1);
+    if (output == NULL) {
+        PyErr_SetString(PyExc_MemoryError, "out of memory");
+        return NULL;
+    }
     outlen = lzf_compress(input, inlen, output, outlen + 1);
 
     if (outlen)
